import json
import os
from pathlib import Path
from typing import Dict, Optional

import nox
from nox import Session

from . import _nox_logger  # noqa, silence logger
from ._env import get_package_name


def _login_lamin_user(user_email: str, env: Optional[Dict[str, str]] = None):
    import boto3
    import lamindb_setup as ln_setup

    if env is not None:
        os.environ.update(env)
    session = boto3.session.Session()
    client = session.client(
        service_name="secretsmanager",
        region_name="us-east-1",
    )
    arn = (
        "arn:aws:secretsmanager:us-east-1:586130067823:secret:laminlabs-internal-sZj1MU"
    )
    secrets = json.loads(client.get_secret_value(SecretId=arn)["SecretString"])
    if user_email == "testuser1@lamin.ai":
        ln_setup.login(user_email, key=secrets["LAMIN_TESTUSER1_API_KEY"])
    elif user_email == "testuser2@lamin.ai":
        ln_setup.login(user_email, key=secrets["LAMIN_TESTUSER2_API_KEY"])
    else:
        raise NotImplementedError


def login_testuser1(session: Session, env: Optional[Dict[str, str]] = None):
    _login_lamin_user("testuser1@lamin.ai", env=env)


def login_testuser2(session: Session, env: Optional[Dict[str, str]] = None):
    _login_lamin_user("testuser2@lamin.ai", env=env)


def run_pre_commit(session: Session):
    if nox.options.default_venv_backend == "none":
        session.run(*"pip install pre-commit".split())
    else:
        session.install("pre-commit")
    session.run("pre-commit", "install")
    session.run("pre-commit", "run", "--all-files")


def run_pytest(session: Session, coverage: bool = True, env: Optional[Dict] = None):
    package_name = get_package_name()
    coverage_args = (
        f"--cov={package_name} --cov-append --cov-report=term-missing".split()
    )
    session.run(
        "pytest",
        "-s",
        *coverage_args,
        env=env,
    )
    if coverage:
        session.run("coverage", "xml")


def build_docs(session: Session, strict: bool = False, strip_prefix: bool = False):
    prefix = "." if Path("./lndocs").exists() else ".."
    if nox.options.default_venv_backend == "none":
        session.run(*f"uv pip install --system {prefix}/lndocs".split())
    else:
        session.install(f"{prefix}/lndocs")
    # do not simply add instance creation here
    args = ["lndocs"]
    if strict:
        args.append("--strict")
    if strip_prefix:
        args.append("--strip-prefix")
    session.run(*args)



import os
import warnings
from pathlib import Path
from subprocess import run
from zipfile import ZipFile

from lamin_utils import logger

from ._env import get_package_name


def zip_docs_dir(zip_filename: str) -> None:
    with ZipFile(zip_filename, "w") as zf:
        zf.write("README.md")
        for f in Path("./docs").glob("**/*"):
            if ".ipynb_checkpoints" in str(f):
                continue
            if f.suffix in {".md", ".ipynb", ".png", ".jpg", ".svg"}:
                zf.write(f, f.relative_to("./docs"))  # add at root level


def zip_docs():
    package_name = get_package_name()
    zip_filename = f"{package_name}_docs.zip"
    zip_docs_dir(zip_filename)
    return package_name, zip_filename


def upload_docs_artifact_aws() -> None:
    package_name, zip_filename = zip_docs()
    run(
        f"aws s3 cp {zip_filename} s3://lamin-site-assets/docs/{zip_filename}",
        shell=True,
    )


def upload_docs_artifact_lamindb() -> None:
    package_name, zip_filename = zip_docs()

    import lamindb as ln

    ln.setup.load("testuser1/lamin-site-assets", migrate=True)

    transform = ln.add(ln.Transform, name=f"CI {package_name}")
    ln.track(transform=transform)

    file = ln.select(ln.File, key=f"docs/{zip_filename}").one_or_none()
    if file is not None:
        file.replace(zip_filename)
    else:
        file = ln.File(zip_filename, key=f"docs/{zip_filename}")
    ln.add(file)


def upload_docs_artifact(aws: bool = False) -> None:
    if os.getenv("GITHUB_EVENT_NAME") not in {"push", "repository_dispatch"}:
        logger.info("Only upload docs artifact for push event.")
        return None

    if aws:
        upload_docs_artifact_aws()
    else:
        try:
            # this is super ugly but necessary right now
            # we might need to close the current instance as it might be corrupted
            import lamindb_setup

            lamindb_setup.close()

            import lamindb as ln  # noqa

            upload_docs_artifact_lamindb()

        except ImportError:
            warnings.warn("Fall back to AWS")
            upload_docs_artifact_aws()



import os
import shutil

from ._env import load_project_yaml


def move_built_docs_to_slash_project_slug():
    if os.environ["GITHUB_EVENT_NAME"] != "push":
        return
    yaml = load_project_yaml()
    shutil.move("_build/html", "_build/html_tmp")
    os.makedirs("_build/html")
    shutil.move("_build/html_tmp", f"_build/html/{yaml['project_slug']}")


def move_built_docs_to_docs_slash_project_slug():
    if os.environ["GITHUB_EVENT_NAME"] != "push":
        return
    yaml = load_project_yaml()
    shutil.move("_build/html", f"_build/{yaml['project_slug']}")
    os.makedirs("_build/html/docs")
    shutil.move(f"_build/{yaml['project_slug']}", "_build/html/docs")



import nox

nox.options.default_venv_backend = "none"


@nox.session
def lint(session: nox.Session) -> None:
    session.run(*"pip install pre-commit".split())
    session.run("pre-commit", "install")
    session.run("pre-commit", "run", "--all-files")


@nox.session
def build(session):
    session.run(*"pip install .[dev]".split())
    session.run(
        "pytest",
        "-s",
        "--cov=laminci",
        "--cov-append",
        "--cov-report=term-missing",
    )
